<?php
/**
 *
 * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 *
 * WARNING:	THERE ARE NO EDITABLE PORTIONS OF THIS SCRIPT.
 * 			ALL OPTIONS ARE CONFIGURABLE VIA THE WEB INTERFACE.
 *
 * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 *
 * Script Name: ImportBuddy.php for use with BackupBuddy backups.
 * Plugin URI: http://pluginbuddy.com/backupbuddy/
 * Description: Backup - Restore - Migrate. Backs up files, settings, and content for a complete snapshot of your site. Allows migration to a new host or URL.
 * Version: 2.2.4 - Iteration 185 - 07/12/2011
 * Author: Dustin Bolton (PluginBuddy.com)
 * Author URI: http://dustinbolton.com/
 *
 * Usage:
 * 
 * 1. Upload this script to the server you would like to restore/migrate to.
 * 2. Upload your backup ZIP file created with BackupBuddy.
 * 3. Navigate to the web address of this script. Ex: http://yoursite.com/importbuddy.php
 * 4. Follow the on screen instructions.
 * 
 */
 
$php_minimum = '5.1'; // User's PHP must be equal or newer to this version.

if ( version_compare( PHP_VERSION, $php_minimum ) < 0 ) {
	die( 'ERROR #9013. See <a href="http://ithemes.com/codex/page/BackupBuddy:_Error_Codes#9013">this codex page for details</a>. Sorry! PHP version ' . $php_minimum . ' or newer is required for BackupBuddy to properly run. You are running PHP version ' . PHP_VERSION . '.' );
}

if (!class_exists("pluginbuddy_importbuddy")) {
	class pluginbuddy_importbuddy {
		var $_version = '2.2.4';
		
		var $_php_minimum = '5.2';
		var $debug = false;										// Displays PHP warnings.
		
		var $_timestamp = 'M j, Y, H:i:s';						// PHP timestamp format.
		var $_total_steps = '6';
		var $_defaults = array(
			'password'					=>		'#PASSWORD#',	// MD5 hash of the import password. Prevents unauthorized access. Default: #PASSWORD#
			'file'						=>		'',				// Selected backup import file.
			'zip_id'					=>		'',				// ID from the random strings in the ZIP filename. This allows finding of the temp directory in the wp-uploads directory.
			'type'						=>		'',				// Set on step 4 via a POST. Valid values: migrate, restore
			
			'skip_files'				=>		false,
			'skip_database_import'		=>		false,
			'skip_database_migration'	=>		false,
			'wipe_database'				=>		false,
			'skip_htaccess'				=>		false,
			'force_compatibility_medium'=>		false,
			'force_compatibility_slow'	=>		false,
			'show_php_warnings'			=>		false,
			'replace_existing_tables'	=>		false,
			
			'log_level'					=>		2,				// 0 = none, 1 = errors only, 2 = errors + warnings, 3 = debugging (all kinds of actions)
			'db_server'					=>		'',
			'db_user'					=>		'',
			'db_password'				=>		'',
			'db_name'					=>		'',
			'db_prefix'					=>		'',
			'siteurl'					=>		'',
		);
		
		var $_backupdata;
		
		const UPLOAD_ACCESS_DENIED = 'To prevent unauthorized file uploads an importbuddy password must be configured to use this feature.';
		
		
		/**
		 *	pluginbuddy_importbuddy()
		 *
		 *	Default constructor.
		 *
		 */
		function pluginbuddy_importbuddy() {
			// Prevent access to importbuddy.php if it is still in plugin directory.
			if ( file_exists( dirname( __FILE__ ) . '/backupbuddy.php' ) ) {
				echo 'The BackupBuddy importer, ImportBuddy, can ONLY be accessed on the destination server that you wish to import your backup to.<br>';
				echo 'Upload the importer in the root web directory on the destination server and try again.<br><br>';
				echo 'If you need assistance visit <a href="http://pluginbuddy.com">http://pluginbuddy.com</a>';
				die();
			}
			
			define( 'ABSPATH', dirname( __FILE__ ) );
			
			// Unpack importbuddy files into importbuddy directory.
			if ( !file_exists( ABSPATH . '/importbuddy' ) ) {
				unpack_importbuddy();
			}
			
			// Return image if requested.
			if ( isset( $_GET['ezimg'] ) ) {
				require_once( 'importbuddy/classes/ezimg.php' );
				ezimg::showImg( $_GET['ezimg'] );
			}
			
			// Start logging time for steps that report how long they took.
			$this->time_start = microtime( true );
			
			// Database step's AJAX-based tester.
			if ( isset( $_POST['action'] ) && ( $_POST['action'] == 'mysql_test' ) ) {
				require_once( 'importbuddy/classes/mysql_test.php' );
			}
			
			// Try to prevent browser timeouts. Greedy script limits are handled on the steps that need them.
			header( 'Keep-Alive: 3600' );
			header( 'Connection: keep-alive' );
			
			// Set up options.
			if ( isset( $_POST['options'] ) ) {
				$this->_options = unserialize( stripslashes( htmlspecialchars_decode( $_POST['options'] ) ) );
				$this->_options = array_merge( $this->_defaults, (array)$this->_options ); // Add in any defaults not explicitly set yet.
			} else {
				$this->_options = $this->_defaults;
			}
			
			// Set up PHP error levels.
			if ( ( $this->debug === true ) || ( $this->_options['show_php_warnings'] === true ) ) {
				error_reporting( E_ERROR | E_WARNING | E_PARSE | E_NOTICE ); // HIGH
				$this->log( 'PHP error reporting set HIGH.' );
			} else {
				error_reporting( E_ALL ^ E_NOTICE ); // LOW
			}
			
			// Detect max execution time for database steps so they can pause when needed for additional PHP processes.
			$this->detected_max_execution_time = str_ireplace( 's', '', ini_get( 'max_execution_time' ) );
			if ( is_numeric( $this->detected_max_execution_time ) === false ) {
				$detected_max_execution_time = 30;
			}
			
			// Determine the current step.
			if ( ( isset( $_GET['step'] ) ) && ( is_numeric( $_GET['step'] ) ) ) {
				$this->_step = $_GET['step'];
			} else {
				$this->_step = 1;
			}
			
			// Run function for the requested step.
			require_once( 'importbuddy/classes/ezimg.php' );
			
			// Handles displaying the current page and running the needed code for that step.
			require_once( 'importbuddy/classes/view_page.php' );
		}
		
		
		/**
		 *	log()
		 *
		 *	Logs to a text file depending on settings.
		 *	0 = none, 1 = errors only, 2 = errors + warnings, 3 = debugging (all kinds of actions)
		 *
		 *	$text		string		Text to log.
		 *	$log_type	string		Valid options: error, warning, all (default so may be omitted).
		 *
		 */
		function log( $text, $log_type = 'all' ) {
			$write = false;
			
			if ( $this->_options['log_level'] == 0 ) { // No logging.
				return;
			} elseif ( $this->_options['log_level'] == 1 ) { // Errors only.
				if ( $log_type == 'error' ) {
					$write = true;
				}
			} elseif ( $this->_options['log_level'] == 2 ) { // Errors and warnings only.
				if ( ( $log_type == 'error' ) || ( $log_type == 'warning' ) ) {
					$write = true;
				}
			} elseif ( $this->_options['log_level'] == 3 ) { // Log all; Errors, warnings, actions, notes, etc.
				$write = true;
			}
			if ( is_writable( ABSPATH . '/importbuddy.txt' ) ) {
				$fh = fopen( ABSPATH . '/importbuddy.txt', 'a');
				if ( $fh !== false ) {
					fwrite( $fh, '[' . date( $this->_timestamp, time() ) . '-' . $log_type . '] ' . $text . "\n" );
					fclose( $fh );
				} else {
					// Don't use alert here since it could recursively look writing to the log.
					echo 'Warning: Unable to write to log file. Verify write permissions to this directory.' ;
				}
			}
		}
		
		
		/**
		 *	tip()
		 *
		 *	Displays a message to the user when they hover over the question mark. Gracefully falls back to normal tooltip.
		 *	HTML is supposed within tooltips.
		 *
		 *	$message		string		Actual message to show to user.
		 *	$title			string		Title of message to show to user. This is displayed at top of tip in bigger letters. Default is blank. (optional)
		 *	$echo_tip		boolean		Whether to echo the tip (default; true), or return the tip (false). (optional)
		 */
		function tip( $message, $title = '', $echo_tip = true ) {
			$tip = ' <a class="pluginbuddy_tip" title="' . $title . ' - ' . $message . '">' . ezimg::genImageTag( 'pluginbuddy_tip.png' ) . '</a>';
			if ( $echo_tip === true ) {
				echo $tip;
			} else {
				return $tip;
			}
		}
		
		
		/**
		 *	alert()
		 *
		 *	Displays a message to the user.
		 *
		 *	$message		string		Message you want to display to the user.
		 *	$error			boolean		OPTIONAL! true indicates this alert is an error and displays as red. Default: false
		 *	$error_code		int			OPTIONAL! Error code number to use in linking in the wiki for easy reference.
		 */
		function alert( $message_title, $message_details = '', $error_code = '' ) {
			?>
			<div class="alert">
				<img src="?ezimg=bullet_error.png" style="float: left;">
				<div style="margin-left: 20px;">
					<b><?php echo $message_title; ?></b><br><br>
					<?php echo $message_details; ?>
					<?php
					if ( $error_code != '' ) {
						echo '<p><a href="http://ithemes.com/codex/page/BackupBuddy:_Error_Codes#' . $error_code . '" target="_new"><i>BackupBuddy Error Code ' . $error_code . ' - Click for more details.</i></a></p>';
						$this->log( 'Error #' . $error_code . ': ' . $message_title, 'error' );
					}
					?>
				</div>
			</div>
			<?php
		}
		
		
		/**
		 *	status_box()
		 *
		 *	Displays a textarea for placing status text into.
		 *
		 *	@param			$default_text	string		First line of text to display.
		 *	@return							string		HTML for textarea.
		 */
		function status_box( $default_text = '' ) {
			return '<textarea style="width: 100%; height: 120px;" id="importbuddy_status">' . $default_text . '</textarea>';
		}		
		
		
		/**
		 *	status()
		 *
		 *	Write a status line into an existing textarea created with the status_box() function.
		 *
		 *	@param			$type		string		message, details, error, or warning. Currently not in use.
		 *	@param			$message	string		Message to append to the status box.
		 *	@return			null
		 */
		function status( $type, $message ) {
			$message = htmlentities( $message );
			$status = date( $this->_timestamp, time() ) . ': ' . $message;
			
			echo '<script type="text/javascript">jQuery( "#importbuddy_status" ).append( "\n' . $status . '");	textareaelem = document.getElementById( "importbuddy_status" );	textareaelem.scrollTop = textareaelem.scrollHeight;	</script>';
			flush();
			
			if ( $type == 'error' ) {
				$this->log( $message, 'error' );
			} elseif ( $type == 'warning' ) {
				$this->log( $message, 'warning' );
			} else {
				$this->log( '[' . $type . ']' . $message, 'all' );
			}
		}
		
		
		/**
		 *	set_greedy_script_limits()
		 *
		 *	Sets greedy script limits to help prevent timeouts, running out of memory, etc.
		 *
		 *	@return		null
		 *
		 */
		function set_greedy_script_limits( &$status_callback = '' ) {
			$status_callback = &$status_callback;
			
			if ( ( $status_callback == '' ) && ( method_exists( $this, 'status' ) ) ) {
				$status_callback = &$this;
			}
			
			$this->status( 'message', 'Requisitioning increased server resources.' );
			
			// Don't abort script if the client connection is lost/closed
			@ignore_user_abort( true );
			
			// Set socket timeout to 2 hours.
			@ini_set( 'default_socket_timeout', 60 * 60 * 2 );
			
			// Set maximum runtime to 2 hours.
			$original_maximum_runtime = ini_get( 'max_execution_time' );
			@set_time_limit( 60 * 60 * 2 );
			$this->log( 'Attempted to increase maximum PHP runtime. Original: ' . $original_maximum_runtime . '; New: ' . @ini_get( 'max_execution_time' ) . '.' );
			
			// Increase the memory limit
			$current_memory_limit = trim( @ini_get( 'memory_limit' ) );
			
			// Make sure a minimum memory limit of 256MB is set.
			if ( preg_match( '/(\d+)(\w*)/', $current_memory_limit, $matches ) ) {
				$current_memory_limit = $matches[1];
				$unit = $matches[2];
				// Up memory limit if currently lower than 256M.
				if ( 'g' !== strtolower( $unit ) ) {
					if ( ( $current_memory_limit < 256 ) || ( 'm' !== strtolower( $unit ) ) )
						@ini_set('memory_limit', '256M');
						$this->log( 'Set memory limit to 256M. Previous value < 256M.' );
				}
			} else {
				// Couldn't determine current limit, set to 256M to be safe.
				@ini_set('memory_limit', '256M');
				$this->log( 'Set memory limit to 256M. Previous value unknown.' );
			}
			$this->log( 'Original PHP memory limit: ' . $current_memory_limit . '; New: ' . @ini_get( 'memory_limit' ) . '.' );
		}
		
		
		function remove_file( $file, $description, $error_on_missing = false ) {
			$this->status( 'message', 'Deleting `' . $description . '`...' );

			@chmod( $file, 0755 ); // High permissions to delete.
			
			if ( is_dir( $file ) ) { // directory.
				$this->remove_dir( $file );
				if ( file_exists( $file ) ) {
					$this->status( 'error', 'Unable to delete directory: `' . $description . '`. You should manually delete it.' );
				} else {
					$this->status( 'message', 'Deleted.' );
				}
			} else { // file
				if ( file_exists( $file ) ) {
					if ( @unlink( $file ) != 1 ) {
						$this->status( 'error', 'Unable to delete file: `' . $description . '`. You should manually delete it.' );
					} else {
						$this->status( 'message', 'Deleted.' );
					}
				}
			}
		}
		
		
		function get_fileperms( $file ) {
			$perms = fileperms( $file );

			if (($perms & 0xC000) == 0xC000) {
				// Socket
				$info = 's';
			} elseif (($perms & 0xA000) == 0xA000) {
				// Symbolic Link
				$info = 'l';
			} elseif (($perms & 0x8000) == 0x8000) {
				// Regular
				$info = '-';
			} elseif (($perms & 0x6000) == 0x6000) {
				// Block special
				$info = 'b';
			} elseif (($perms & 0x4000) == 0x4000) {
				// Directory
				$info = 'd';
			} elseif (($perms & 0x2000) == 0x2000) {
				// Character special
				$info = 'c';
			} elseif (($perms & 0x1000) == 0x1000) {
				// FIFO pipe
				$info = 'p';
			} else {
				// Unknown
				$info = 'u';
			}
			
			// Owner
			$info .= (($perms & 0x0100) ? 'r' : '-');
			$info .= (($perms & 0x0080) ? 'w' : '-');
			$info .= (($perms & 0x0040) ?
						(($perms & 0x0800) ? 's' : 'x' ) :
						(($perms & 0x0800) ? 'S' : '-'));
			
			// Group
			$info .= (($perms & 0x0020) ? 'r' : '-');
			$info .= (($perms & 0x0010) ? 'w' : '-');
			$info .= (($perms & 0x0008) ?
						(($perms & 0x0400) ? 's' : 'x' ) :
						(($perms & 0x0400) ? 'S' : '-'));
			
			// World
			$info .= (($perms & 0x0004) ? 'r' : '-');
			$info .= (($perms & 0x0002) ? 'w' : '-');
			$info .= (($perms & 0x0001) ?
						(($perms & 0x0200) ? 't' : 'x' ) :
						(($perms & 0x0200) ? 'T' : '-'));
			
			return $info;
		}
		
		/**
		 *	notify_existing_wordpress()
		 *
		 *	Notifies the user with an alert if WordPress appears to already exist in this directory.
		 *
		 *	@return			null
		 */
		function notify_existing_wordpress() {
			if ( file_exists( ABSPATH . '/wp-config.php' ) ) {
				$this->log( 'Found existing WordPress installation.', 'warning' );
				//echo '<br><br>';
				$this->alert( 'WARNING: Existing WordPress installation found.', 'It is recommended that existing WordPress files and database be removed prior to migrating or restoring to avoid conflicts. You should not install WordPress prior to migrating.' );
			}
		}
		
		
		/**
		 *	get_zip_id()
		 *
		 *	Given a BackupBuddy ZIP file, extracts the random ZIP ID from the filename. This random string determines
		 *	where BackupBuddy will find the temporary directory in the backup's wp-uploads directory. IE a zip ID of
		 *	3poje9j34 will mean the temporary directory is wp-uploads/temp_3poje9j34/. backupbuddy_dat.php is in this
		 *	directory as well as the SQL dump.
		 *
		 *	Currently handles old BackupBuddy ZIP file format. Remove this backward compatibility at some point.
		 *
		 *	$file			string		BackupBuddy ZIP filename.
		 *	@return			string		ZIP ID characters.
		 *
		 */
		function get_zip_id( $file ) {
			$posa = strrpos($file,'_')+1;
			$posb = strrpos($file,'-')+1;
			if ( $posa < $posb ) {
				$this->_options['zip_id'] = $posb;
				$this->_options['zip_id'] = strrpos($file,'-')+1;
			} else {
				$this->_options['zip_id'] = $posa;
				$this->_options['zip_id'] = strrpos($file,'_')+1;
			}
			$this->_options['zip_id'] = substr( $this->_options['file'], $this->_options['zip_id'], strlen($this->_options['file'])-$this->_options['zip_id']-4 );
			
			return $this->_options['zip_id'];
		}
		
		
		/**
		 *	load_backup_dat()
		 *
		 *	Gets the serialized data from the backupbuddy_dat.php file inside of the backup ZIP.
		 *	This happens post-file-extraction.
		 *
		 *	Saves data to $this->_backupdata.
		 *
		 *	@return			null
		 *
		 */
		function load_backup_dat() {
			$this->log( 'STARTING Loading backup dat file....' );
			$backupdata_file = ABSPATH . '/wp-content/uploads/temp_'. $this->_options['zip_id'] .'/backupbuddy_dat.php'; // Full backup dat file location
			$backupdata_file_new = ABSPATH . '/wp-content/uploads/backupbuddy_temp/'. $this->_options['zip_id'] .'/backupbuddy_dat.php'; // Full backup dat file location
			
			if ( file_exists( $backupdata_file ) ) { // Full backup location.
				$backupdata = file_get_contents( $backupdata_file );
			} elseif ( file_exists( $backupdata_file_new ) ) { // Full backup location.
				$backupdata = file_get_contents( $backupdata_file_new );
			} elseif ( file_exists( ABSPATH . '/backupbuddy_dat.php' ) ) { // DB only location.
				$backupdata = file_get_contents( ABSPATH . '/backupbuddy_dat.php' );
			} else { // Missing.
				$this->alert( 'Error #9003: BackupBuddy data file (backupbuddy_dat.php) missing or unreadable.', 'There may be a problem with the backup file, the files could not be extracted (you may manually extract the zip file in this directory to manually do this portion of restore), or the files were deleted before this portion of the restore was reached.  Start the import process over or try manually extracting (unzipping) the files then starting over. Restore will not continue to protect integrity of any existing data.', true, '9003' );
				die( ' Halted.' );
			}
			
			// Unserialize data; If it fails it then decodes the obscufated data then unserializes it. (new dat file method starting at 2.0).
			if ( !is_serialized( $backupdata ) || ( false === ( $this->_backupdata = unserialize( $backupdata ) ) ) ) {
				// Skip first line.
				$second_line_pos = strpos( $backupdata, "\n" ) + 1;
				$backupdata = substr( $backupdata, $second_line_pos );
				
				// Decode back into an array.
				$this->_backupdata = unserialize( base64_decode( $backupdata ) );
			}
			
			$this->log( 'DONE Loading backup dat file.' );
		}
		
		
		/**
		 *	migrate_htaccess()
		 *
		 *	Migrates .htaccess file if it exists.
		 *
		 *	@return		boolean		True on success. Currently always true.
		 *
		 */
		function migrate_htaccess() {
			// If there is no .htaccess file then return.
			if ( !file_exists( ABSPATH . '/.htaccess' ) ) {
				$this->log( 'No htaccess file found. Skipping.' );
				return;
			}
			if ( !is_writable( ABSPATH . '/.htaccess' ) ) {
				$this->log( 'Permissions prevent writing to .htaccess file. Skipping.' );
				$this->alert( 'Unable to write to .htaccess file.', 'Permissions are preventing your .htaccess file from being modified.', '9020' );
				$this->alert( 'ERROR: Unable to write to file .htaccess.', 'Verify this file has proper write permissions.', '9020' );
				return;
			}
			
			$this->log( 'Migrating htaccess file.' );
			
			$oldurl = strtolower( $this->_backupdata['siteurl'] );
			$oldurl = str_replace( '/', '\\', $oldurl );
			$oldurl = str_replace( 'http:\\', '', $oldurl );
			$oldurl = trim( $oldurl, '\\' );
			$oldurl = explode( '\\', $oldurl );
			$oldurl[0] = '';
			
			$newurl = strtolower( $this->_options['siteurl'] );
			$newurl = str_replace( '/', '\\', $newurl );
			$newurl = str_replace( 'http:\\', '', $newurl );
			$newurl = trim( $newurl, '\\' );
			$newurl = explode( '\\', $newurl );
			$newurl[0] = '';
			
			$this->status( 'message', 'Checking .htaccess file.' );
			
			// If the URL (domain and/or URL subdirectory ) has changed, then need to update .htaccess file.
			if ( $newurl !== $oldurl ) {
				$this->status( 'message', 'URL directory has changed. Updating from `' . $oldurl . '` to `' . $newurl . '`.' );
				
				$rewrite_lines = array();
				$got_rewrite = false;
				$rewrite_path = implode( '/', $newurl );
				$file_array = file( ABSPATH . '/.htaccess' );
				
				foreach ($file_array as $line_number => $line) {
					if ( $got_rewrite == true ) { // In a WordPress section.
						if ( strstr( $line, 'END WordPress' ) ) { // End of a WordPress block so stop replacing.
							$got_rewrite = false;
							$rewrite_lines[] =  $line; // Captures end of WordPress block.
						} else {
							if ( strstr( $line, 'RewriteBase' ) ) { // RewriteBase
								$rewrite_lines[] = 'RewriteBase ' . $rewrite_path . '/' . "\n";
							} elseif ( strstr( $line, 'RewriteRule' ) ) { // RewriteRule
								if ( strstr( $line, '^index\.php$' ) ) { // Handle new strange rewriterule. Leave as is.
									$rewrite_lines[] = $line;
									$this->log( 'Htaccess ^index\.php$ detected. Leaving as is.' );
								} else { // Normal spot.
									$rewrite_lines[] = 'RewriteRule . ' . $rewrite_path . '/index.php' . "\n";
								}
							} else {
								$rewrite_lines[] =  $line; // Captures everything inside WordPress block we arent modifying.
							}
						}
					} else { // Outside a WordPress section.
						if ( strstr( $line, 'BEGIN WordPress' ) ) {
							$got_rewrite = true; // Beginning of a WordPress block so start replacing.
						}
						$rewrite_lines[] =  $line; // Captures everything outside of WordPress block.
					}
				}
					
				$handling = fopen( ABSPATH . '/.htaccess', 'w');
				fwrite( $handling, implode( $rewrite_lines ) );
				fclose( $handling );
				unset( $handling );
				
				$this->status( 'message', 'Finished migrating .htaccess file.' );
			} else {
				$this->status( 'message', 'No changes needed for .htaccess file.' );
			}
			
			return true;
		}
		
		
		/**
		 *	connect()
		 *
		 *	Initializes a connection to the mysql database.
		 *
		 *	@return		boolean		True on success; else false. Success testing is very loose.
		 */
		function connect_database() {
			// Set up database connection.
			if ( false === @mysql_connect( $this->_options['db_server'], $this->_options['db_user'], $this->_options['db_password'] ) ) {
				$this->alert( 'ERROR: Unable to connect to database server and/or log in. Verify the database server name, username, and password. Details: ' . mysql_error(), true, '9006' );
				return false;
			}
			$database_name = mysql_real_escape_string( $this->_options['db_name'] );
			
			flush();
			
			// Select the database.
			if ( false === @mysql_select_db( $this->_options['db_name'] ) ) {
				$this->alert( 'ERROR: Unable to select your specified database. Verify the database name and that you have set up proper permissions for your specified username to access it. Details: ' . mysql_error(), true, '9007' );
				return false;
			}
			
			// Set up character set. Important.
			mysql_query("SET NAMES 'utf8'");
			
			return true;
		}
		
		
		/**
		 *	restore_database()
		 *
		 *	Preset the following variables within $this->_options before continuing:
		 *		db_server, db_user, db_password, db_name
		 *
		 *	@return			mixed		true: success, false: failure, integer: query to start next db import chunk on
		 */
		function restore_database( $query_start = 0 ) {
			$this->status( 'message', 'Beginning database import...' );
			
			$this->connect_database();
			
			// Require a table prefix.
			if ( $this->_options['db_prefix'] == '' ) {
				$this->alert( 'ERROR: A database prefix is required for importing. Details: ' . mysql_error(), true, '9008' );
			}
			
			if ( $query_start > 0 ) {
				$this->status( 'message', 'Continuining to restore database dump starting at query ' . $query_start . '.' );
			} else {
				$this->status( 'message', 'Restoring database dump. This may take a moment...' );
			}
			
			flush();
			
			// Check number of tables already existing with this prefix. Skips this check on substeps of DB import.
			if ( $query_start == 0 ) {
				$result = mysql_query( "SHOW TABLES LIKE '" . mysql_real_escape_string( $this->_options['db_prefix'] ) . "%'" );
				if ( mysql_num_rows( $result ) > 0 ) {
					//echo ezimg::genImageTag('bullet_error.png').' Found ' . mysql_num_rows( $result ) . ' existing tables with same prefix ... Restore stopped to prevent accidental overwrite of existing data.';
					$this->status( 'error', 'Database import halted to prevent overwriting existing WordPress data.' );
					$this->alert( 'Database import halted to prevent overwriting existing WordPress data.', 'The database already contains a WordPress installation with this prefix (' . mysql_num_rows( $result ) . ' tables). Restore has been stopped to prevent overwriting existing data. Please go back and enter a new database name and/or prefix OR select the option to wipe the database prior to import from the advanced settings on the first import step.', '9014' );
					return false;
				}
				unset( $result );
			}
			
			
			// Import SQL dump onto new server. NOTE: This data has NOT been migrated. It is identical to the source server still at this point.
			$import_result = $this->import_sql_dump( $query_start );
			
			// CLEANUP
			mysql_close();
			
			$this->status( 'message', 'Database import complete.' );
			
			return $import_result;
		}
		
		
		/**
		 *	import_sql_dump()
		 *
		 *	Directly inserts the source SQL dump into the new database.
		 *	Does NOT modify any data or do any migration.
		 *
		 *	@return		boolean		True: success, False: failed.
		 *
		 */
		function import_sql_dump( $query_start = 0 ) {
			// TODO: debugging
			//$query_start = 839;
			$this->status( 'message', 'Starting import of SQL date... This may take a moment...' );
			
			$file_stream = false; // Default state.
			if ( file_exists ( ABSPATH . '/wp-content/uploads/temp_' . $this->_options['zip_id'] . '/db.sql' ) ) { // Full backup found.
				$file_stream = fopen( ABSPATH . '/wp-content/uploads/temp_'.$this->_options['zip_id'].'/db.sql', 'r' );
			} elseif ( file_exists ( ABSPATH . '/db.sql' ) ) { // DB-only backup found.
				$file_stream = fopen( ABSPATH . '/db.sql', 'r' );
			} elseif ( file_exists ( ABSPATH . '/wp-content/uploads/backupbuddy_temp/' . $this->_options['zip_id'] . '/db_1.sql' ) ) { // Full backup found. 2.0 method.
				$file_stream = fopen( ABSPATH . '/wp-content/uploads/backupbuddy_temp/' . $this->_options['zip_id'] . '/db_1.sql', 'r' );
			} elseif ( file_exists ( ABSPATH . '/db_1.sql' ) ) { // DB-only backup found. 2.0 method.
				$file_stream = fopen( ABSPATH . '/db_1.sql', 'r' );
			}
			
			if ( false === $file_stream ) {
				$this->alert( 'ERROR: Unable to find any database backup data in the selected backup.', true, '9009' );
				$this->status( 'error', 'Unable to find any database backup data in the selected backup. Error #9009.' );
				return false;
			}
			
			// Iterate through each full row action and import it one at a time.
			
			$query_count = 0;
			$file_data = '';
			
			while ( ! feof( $file_stream ) ) {
			
				while ( false === strpos( $file_data, "\n" ) ) {
					$file_data .= fread( $file_stream, 4096 );
				}
				
				$queries = explode( "\n", $file_data );
				
				if ( preg_match( "/\n$/", $file_data ) ) {
					$file_data = '';
				} else {
					$file_data = array_pop( $queries );
				}
				
				// TODO: DEBUGGING:
				//$this->_options['max_execution_time'] = 0.41;
				
				// Loops through each full query.
				foreach ( (array) $queries as $query ) {
					if ( $query_count < ( $query_start - 1 ) ) { // Handle skipping any queries up to the point we are at.
						$query_count++;
						continue; // Continue to next foreach iteration.
					} else {
						$query_count++;
					}
					
					$query = trim( $query );
					
					if ( empty( $query ) ) {
						continue;
					}
					
					$result = $this->import_sql_dump_line( $query );
					
					if ( false === $result ) { // Skipped query
						continue;
					}
					
					if ( 0 === ( $query_count % 500 ) ) {
						$this->status( 'message', 'Working...' );
					}
					/*
					if ( 0 === ( $query_count % 6000 ) ) {
						echo "<br>\n";
					}
					*/
					
					// If we are within 1 second of reaching maximum PHP runtime then stop here so that it can be picked up in another PHP process...
					if ( ( ( microtime( true ) - $this->time_start ) + 1 ) >= $this->_options['max_execution_time'] ) {
					// TODO: Debugging:
					//if ( ( ( microtime( true ) - $this->time_start ) ) >= $this->_options['max_execution_time'] ) {
						$this->status( 'message', 'Exhausted available PHP time to import for this page load. Last query: ' . $query_count . '.' );
						
						fclose( $file_stream );
						
						return ( $query_count + 1 );
						//break 2;
					}
				}
				
			}
			
			fclose( $file_stream );
			
			$this->status( 'message', 'Import of SQL data complete.' );			
			$this->status( 'message', 'Took ' . round( microtime( true ) - $this->time_start, 3 ) . ' seconds on ' . $query_count . ' queries. ' );
			
			return true;
		}
		
		
		/**
		 *	import_sql_dump_line()
		 *
		 *	Imports a line/query into the database.
		 *	Handles using the specified table prefix.
		 *
		 *	$query		string		Query string to run for importing.
		 *	@return		boolean		True=success, False=failed.
		 *
		 */
		function import_sql_dump_line( $query ) {
			$old_prefix = $this->_backupdata['db_prefix'];
			$new_prefix = $this->_options['db_prefix'];
			
			$query_operators = 'INSERT INTO|CREATE TABLE|REFERENCES|CONSTRAINT';
			
			// Replace database prefix in query.
			if ( $old_prefix !== $new_prefix ) {
				$query = preg_replace( "/^($query_operators)(\s+`?)$old_prefix/i", "\${1}\${2}$new_prefix", $query ); // 4-29-11
			}
			
			// Run the query
			// Disabled to prevent from running on EVERY line. Now just running before this. mysql_query("SET NAMES 'utf8'"); // Force UTF8
			$result = mysql_query( $query );
			
			if ( false === $result ) {
				$this->alert( 'ERROR: Unable to import SQL query: ' . mysql_error(), true, '9010' );
				return false;
			} else {
				return true;
			}
		}
		
		
		/**
		 *	migrate_database()
		 *
		 *	Migrates the already imported database's content for updates ABSPATH and URL.
		 *
		 *	@return		boolean		True=success, False=failed.
		 *
		 */
		function migrate_database() {
			$this->status( 'message', 'Migrating database content...' );
			$this->log( 'Beginning migration of DB.' );
			$this->log( '_options[] value: ' . serialize( $this->_options ) );
			
			$old_abspath = $this->_backupdata['abspath'];
			//$old_abspath = str_replace( '\\', '\\\\', $this->_backupdata['abspath'] ); // Remove escaping of windows paths. - Caused problems. REMOVE?
			$old_abspath = preg_replace( '|/+$|', '', $old_abspath ); // Remove escaping of windows paths.
			//$new_abspath = str_replace( '\\', '\\\\', ABSPATH ); Caused problems. REMOVE?
			$new_abspath = ABSPATH;
			$this->log( 'ABSPATH change for database ... Old Path: ' . $old_abspath . ', New Path: ' . $new_abspath . '.' );
			
			$old_url = $this->_backupdata['siteurl'];  // the value you want to search for
			
			// If http://www.blah.... then also we will replace http://blah... and vice versa.
			if ( stristr( $old_url, 'http://www.' ) || stristr( $old_url, 'https://www.' ) ) {
				$old_url_alt = str_ireplace( 'http://www.', 'http://', $old_url );
				$old_url_alt = str_ireplace( 'https://www.', 'https://', $old_url_alt );
			} else {
				$old_url_alt = str_ireplace( 'http://', 'http://www.', $old_url );
				$old_url_alt = str_ireplace( 'https://', 'https://www.', $old_url_alt );
			}
			
			$new_url = preg_replace( '|/*$|', '', $this->_options['siteurl'] );  // the value to replace it with
			$this->log( 'URL change for database ... Old URL: ' . $old_url . ', New URL: ' . $new_url . '.' );
			
			$count_tables_checked = 0;
			$count_items_checked = 0;
			$count_items_changed = 0;
			
			// Connect to database.
			$this->connect_database();
			
			flush();
			
			if ( $this->_options['siteurl'] != $this->_options['home'] ) { // Update home url value if it exists since it is custom.
				mysql_query( "UPDATE `" . $this->_options['db_prefix'] . "options` SET option_value='" . mysql_real_escape_string( $this->_options['home'] ) . "' WHERE option_name='home' LIMIT 1" );
			}
			
			$row_loop = 0;
			
			// Loop through the tables matching this prefix. Does NOT change data in other tables.
			// This changes actual data on a column by column basis for very row in every table.
			$tables = array();
			$result = mysql_query( "SHOW TABLES LIKE '" . $this->_options['db_prefix'] . "%'" );
			while ( $table = mysql_fetch_row( $result ) ) {
				$tables[] = $table[0];
			}
			unset( $table );
			$this->status( 'message', 'Found ' . mysql_num_rows( $result ) . ' WordPress tables. ' );
			unset( $result );
			
			
			$bruteforce_excluded_tables = array(
								$this->_options['db_prefix'] . 'posts',
								$this->_options['db_prefix'] . 'users',
								$this->_options['db_prefix'] . 'usermeta',
								$this->_options['db_prefix'] . 'terms',
								$this->_options['db_prefix'] . 'term_taxonomy',
								$this->_options['db_prefix'] . 'term_relationships',
								$this->_options['db_prefix'] . 'postmeta',
								$this->_options['db_prefix'] . 'options',
								$this->_options['db_prefix'] . 'comments',
								$this->_options['db_prefix'] . 'commentmeta',
								$this->_options['db_prefix'] . 'links',
							);
			
			$bruteforce_tables = array_remove( $tables, $bruteforce_excluded_tables ); // Removes all tables listed in $excluded_tables from $tables.
			unset( $tables );

			if ( $this->_options['siteurl'] != $this->_options['home'] ) { // Update home url value if it exists since it is custom.
				$old_urls = array( $old_url, $old_url_alt, $this->_backupdata['homeurl'] );
				$new_urls = array( $new_url, $new_url, $this->_options['home'] );
				if ( empty( $this->_backupdata['homeurl'] ) ) { // old BackupBuddy versions did not store the previous homeurl. Hang onto this for backwards compatibility for a while.
					$this->status( 'error', 'Failed. Your current backup does not include a home URL. Make a new backup with the latest BackupBuddy before migrating.' );
					$this->alert( 'Error #4543849498. Missing old home URL.', 'Failed. Your current backup does not include a home URL. Make a new backup with the latest BackupBuddy before migrating.' );
				} else {
					$this->status( 'message', 'Home URLs will also be updated along with site URLs.' );
				}
			} else { // Site URL updates only.
				$old_urls = array( $old_url, $old_url_alt );
				$new_urls = array( $new_url, $new_url );
			}
			
			$old_fullreplace = array( $old_url, $old_url_alt, $old_abspath );
			$new_fullreplace = array( $new_url, $new_url, $new_abspath );
						
			require_once( 'importbuddy/lib/dbreplace/dbreplace.php' );
			$dbreplace = new pluginbuddy_dbreplace( $this );
			
			// Update site URL strings in posts table for rows post_content, guid.
			$this->status( 'message', 'Updating posts table site URLs.' );
			$dbreplace->text( $this->_options['db_prefix'] . 'posts', $old_urls, $new_urls, array( 'post_content', 'guid', 'post_excerpt', 'post_content_filtered' ) );
			$this->status( 'message', 'Site URLs updated in posts table.' );
			
			// Misc string replacements
			$this->status( 'message', 'Replacing WordPress core database text data…' );
			$dbreplace->text( $this->_options['db_prefix'] . 'users', $old_urls, $new_urls, array( 'user_url' ) );
			$dbreplace->text( $this->_options['db_prefix'] . 'users', $old_urls, $new_urls, array( 'comment_content', 'comment_author_url' ) );
			$dbreplace->text( $this->_options['db_prefix'] . 'links', $old_urls, $new_urls, array( 'link_url', 'link_image', 'link_target', 'link_description', 'link_notes', 'link_rss' ) );
			$this->status( 'message', 'WordPress core database text replaced.' );
			
			// Misc serialized data replacements.
			$this->status( 'message', 'Replacing WordPress core database serialized data...' );
			$dbreplace->serialized( $this->_options['db_prefix'] . 'options', $old_fullreplace, $new_fullreplace, array( 'option_value' ) );
			$dbreplace->serialized( $this->_options['db_prefix'] . 'usermeta', $old_fullreplace, $new_fullreplace, array( 'meta_value' ) );
			$dbreplace->serialized( $this->_options['db_prefix'] . 'postmeta', $old_fullreplace, $new_fullreplace, array( 'meta_value' ) );
			$dbreplace->serialized( $this->_options['db_prefix'] . 'commentmeta', $old_fullreplace, $new_fullreplace, array( 'meta_value' ) );
			$this->status( 'message', 'WordPress core database serialized data replaced.' );
			
			foreach ( $bruteforce_tables as $bruteforce_table ) {
				$dbreplace->bruteforce_table( $bruteforce_table, $old_fullreplace, $new_fullreplace );
			}
			
			
			// Update table prefixes in some WordPress meta data.
			$old_prefix = $this->_backupdata['db_prefix'];
			$new_prefix = mysql_real_escape_string( $this->_options['db_prefix'] );
			if ($old_prefix != $new_prefix ) {
				mysql_query("UPDATE `".$new_prefix."usermeta` SET meta_key = REPLACE(meta_key, '".$old_prefix."', '".$new_prefix."' );");
				mysql_query("UPDATE `".$new_prefix."options` SET option_name = '".$new_prefix."user_roles' WHERE option_name ='".$old_prefix."user_roles' AND blog_id='0';");
				$this->status( 'message', 'Updated prefix META data.' );
			}
			
			$this->status( 'message', 'Migrated ' . count( $bruteforce_tables ) . ' tables via brute force.' );
			$this->status( 'message', 'Took ' . round( microtime( true ) - $this->time_start, 3 ) . ' seconds. Done.' );
			
			$this->status( 'message', 'Database content migrated.' );
			
			return true;
		}
		
		
		/**
		 *	wipe_database()
		 *
		 *	Clear out the existing database to prepare for importing new data.
		 *
		 *	@return			boolean		Currently always true.
		 */
		function wipe_database() {
			$this->status( 'message', 'Beginning wipe of database...' );
			
			// Connect to database.
			$this->connect_database();
			
			$result = mysql_query( 'SHOW TABLES' );
			$table_wipe_count = mysql_num_rows( $result );
			while( $row = mysql_fetch_row( $result ) ) {
				mysql_query( 'DROP TABLE `' . $row[0] . '`' );
			}
			mysql_free_result( $result ); // Free memory.
			$this->status( 'message', 'Wiped database of ' . $table_wipe_count . ' tables.' );
			
			return true;
		}
		
		
		/**
		 *	remove_dir()
		 *
		 *	?
		 *
		 *	@return			?
		 */
		function remove_dir( $dir ) {
			if ( !file_exists( $dir ) ) {
				return true;
			}
			if ( !is_dir( $dir ) || is_link( $dir ) ) {
				return unlink($dir);
			}
			foreach ( scandir( $dir ) as $item ) {
				if ( $item == '.' || $item == '..' ) {
					continue;
				}
				if ( !$this->remove_dir( $dir . "/" . $item ) ) {
					chmod( $dir . "/" . $item, 0777 );
					if ( !$this->remove_dir( $dir . "/" . $item ) ) {
						return false;
					}
				}
			}
			return rmdir($dir);
		}
		
		
		/**
		 *	phpinfo_array()
		 *
		 *	Get phpinfo() data as an array.
		 *
		 *	@return			array		Array of phpinfo() data.
		 */
		function phpinfo_array() {
			ob_start();
			phpinfo(-1);
			
			$pi = preg_replace(
			array('#^.*<body>(.*)</body>.*$#ms', '#<h2>PHP License</h2>.*$#ms',
			'#<h1>Configuration</h1>#',  "#\r?\n#", "#</(h1|h2|h3|tr)>#", '# +<#',
			"#[ \t]+#", '#&nbsp;#', '#  +#', '# class=".*?"#', '%&#039;%',
			'#<tr>(?:.*?)" src="(?:.*?)=(.*?)" alt="PHP Logo" /></a>'
			.'<h1>PHP Version (.*?)</h1>(?:\n+?)</td></tr>#',
			'#<h1><a href="(?:.*?)\?=(.*?)">PHP Credits</a></h1>#',
			'#<tr>(?:.*?)" src="(?:.*?)=(.*?)"(?:.*?)Zend Engine (.*?),(?:.*?)</tr>#',
			"# +#", '#<tr>#', '#</tr>#'),
			array('$1', '', '', '', '</$1>' . "\n", '<', ' ', ' ', ' ', '', ' ',
			'<h2>PHP Configuration</h2>'."\n".'<tr><td>PHP Version</td><td>$2</td></tr>'.
			"\n".'<tr><td>PHP Egg</td><td>$1</td></tr>',
			'<tr><td>PHP Credits Egg</td><td>$1</td></tr>',
			'<tr><td>Zend Engine</td><td>$2</td></tr>' . "\n" .
			'<tr><td>Zend Egg</td><td>$1</td></tr>', ' ', '%S%', '%E%'),
			ob_get_clean());
			
			$sections = explode('<h2>', strip_tags($pi, '<h2><th><td>'));
			unset($sections[0]);
			
			$pi = array();
			foreach($sections as $section){
				$n = substr($section, 0, strpos($section, '</h2>'));
				preg_match_all( '#%S%(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?%E%#', $section, $askapache, PREG_SET_ORDER);
				foreach($askapache as $m) {
					if (isset($m[2])) { // Fix undefined offset warning.
						$pi[$n][$m[1]]=(!isset($m[3])||$m[2]==$m[3])?$m[2]:array_slice($m,2);
					}
				}
			}
			
			return $pi;
		}
		
		
		/**
		 *	migrate_wp_config()
		 *
		 *	Migrates and updates the wp-config.php file contents as needed.
		 *
		 *	@return			null			True on success. Currently always true.
		 */
		function migrate_wp_config() {
			$this->status( 'message', 'Starting migration of wp-config.php file...' );
			
			flush();
			
			// Check that we can write to this file.
			if ( !is_writable( ABSPATH . '/wp-config.php' ) ) {
				$this->alert( 'ERROR: Unable to write to file wp-config.php.', 'Verify this file has proper write permissions.', '9020' );
				return false;
			}
			
			if ( file_exists( ABSPATH . '/wp-config.php' ) ) {
				// Useful REGEX site: http://gskinner.com/RegExr/
				
				$updated_home_url = false;
				$wp_config = array();
				$lines = file( ABSPATH . '/wp-config.php' );
				
				$patterns = array();
				$replacements = array();
				
				/*
				Update WP_SITEURL, WP_HOME if they exist.
				Update database DB_NAME, DB_USER, DB_PASSWORD, and DB_HOST.
				RegExp: /define\([\s]*('|")WP_SITEURL('|"),[\s]*('|")(.)*('|")[\s]*\);/gi
				pattern: define\([\s]*('|")WP_SITEURL('|"),[\s]*('|")(.)*('|")[\s]*\);
				*/
				$pattern[0] = '/define\([\s]*(\'|")WP_SITEURL(\'|"),[\s]*(\'|")(.)*(\'|")[\s]*\);/i';
				$replace[0] = "define( 'WP_SITEURL', '" . trim( $this->_options['siteurl'], '/' ) . "' );";
				$pattern[1] = '/define\([\s]*(\'|")WP_HOME(\'|"),[\s]*(\'|")(.)*(\'|")[\s]*\);/i';
				$replace[1] = "define( 'WP_HOME', '" . trim( $this->_options['home'], '/' ) . "' );";
				
				$pattern[2] = '/define\([\s]*(\'|")DB_NAME(\'|"),[\s]*(\'|")(.)*(\'|")[\s]*\);/i';
				$replace[2] = "define( 'DB_NAME', '" . $this->_options['db_name'] . "' );";
				$pattern[3] = '/define\([\s]*(\'|")DB_USER(\'|"),[\s]*(\'|")(.)*(\'|")[\s]*\);/i';
				$replace[3] = "define( 'DB_USER', '" . $this->_options['db_user'] . "' );";
				$pattern[4] = '/define\([\s]*(\'|")DB_PASSWORD(\'|"),[\s]*(\'|")(.)*(\'|")[\s]*\);/i';
				$replace[4] = "define( 'DB_PASSWORD', '" . $this->_options['db_password'] . "' );";
				$pattern[5] = '/define\([\s]*(\'|")DB_HOST(\'|"),[\s]*(\'|")(.)*(\'|")[\s]*\);/i';
				$replace[5] = "define( 'DB_HOST', '" . $this->_options['db_server'] . "' );";
				
				/*
				Update table prefix.
				RegExp: /\$table_prefix[\s]*=[\s]*('|")(.)*('|");/gi
				pattern: \$table_prefix[\s]*=[\s]*('|")(.)*('|");
				*/
				$pattern[6] = '/\$table_prefix[\s]*=[\s]*(\'|")(.)*(\'|");/i';
				$replace[6] = '$table_prefix = \'' . $this->_options['db_prefix'] . '\';';
				
				// Perform the actual replacement.
				$lines = preg_replace( $pattern, $replace, $lines );
				
				// Write changes to config file.
				if ( false === ( file_put_contents( ABSPATH . '/wp-config.php', $lines ) ) ) {
					$this->alert( 'ERROR: Unable to save changes to wp-config.php.', 'Verify this file has proper write permissions.', '9020' );
					return false;
				}
				
				unset( $lines );
			} else {
				$this->status( 'warning', 'Warning: wp-config.php file not found.' );
				$this->alert( 'Note: wp-config.php file not found. This is normal for a database only backup.' );
			}
			
			$this->status( 'message', 'Migration of wp-config.php complete.' );
			
			return true;
		}
		
		
		/**
		 *	delete_directory()
		 *
		 *	Delete a directory and all of its contents.
		 *
		 *	@param			$dir		string		Path of directory to delete.
		 *	@return			null
		 */
		function delete_directory( $dir ) {
			if ($handle = opendir($dir)) {
				$array = array();
				
				while (false !== ($file = readdir($handle))) {
					if ($file != "." && $file != "..") {
				
						if(is_dir($dir.$file))
						{
							if(!@rmdir($dir.$file)) // Empty directory? Remove it
							{
							//deleteDir($dir.$file.'/'); // Not empty? Delete the files inside it
							}
						}
						else
						{
						   @unlink($dir.$file);
						}
					}
				}
				closedir($handle);
				
				@rmdir($dir);
			}
		}

	}
}
define( 'pluginbuddy_importbuddy', true ); // Tell Server Info page to not load some sections.
$pluginbuddy_importbuddy = new pluginbuddy_importbuddy();


// Compatibility with some WordPress localization.
function __( $text, $domain ) {
	return $text;
}
function _e( $text, $domain ) {
	echo $text;
}


/**
 *	mkdir_recursive()
 *
 *	Recursively creates the directories needed to generate a full directory path.
 *
 *	$path		string		Full absolute path to generate.
 *	@return		null
 *
 */
function mkdir_recursive( $path ) {
	if ( empty( $path ) ) { // prevent infinite loop on bad path
		return;
	}
	is_dir( dirname( $path ) ) || mkdir_recursive( dirname( $path ) );
	return is_dir( $path ) || mkdir( $path );
}


/**
 * Check value to find if it was serialized.
 *
 * If $data is not an string, then returned value will always be false.
 * Serialized data is always a string.
 * Courtesy WordPress; since WordPress 2.0.5.
 *
 * @param mixed $data Value to check to see if was serialized.
 * @return bool False if not serialized and true if it was.
 */
function is_serialized( $data ) {
	// if it isn't a string, it isn't serialized
	if ( ! is_string( $data ) )
		return false;
	$data = trim( $data );
 	if ( 'N;' == $data )
		return true;
	$length = strlen( $data );
	if ( $length < 4 )
		return false;
	if ( ':' !== $data[1] )
		return false;
	$lastc = $data[$length-1];
	if ( ';' !== $lastc && '}' !== $lastc )
		return false;
	$token = $data[0];
	switch ( $token ) {
		case 's' :
			if ( '"' !== $data[$length-2] )
				return false;
		case 'a' :
		case 'O' :
			return (bool) preg_match( "/^{$token}:[0-9]+:/s", $data );
		case 'b' :
		case 'i' :
		case 'd' :
			return (bool) preg_match( "/^{$token}:[0-9.E-]+;\$/", $data );
	}
	return false;
}


/**
 *	array_remove()
 *
 *	Removes array values in $remove from $array.
 *
 *	@param			$array		array		Source array. This will have values removed and be returned.
 *	@param			$remove		array		Array of values to search for in $array and remove.
 *	@return						array		Returns array $array stripped of all values found in $remove
 */
function array_remove( $array, $remove ) {
	if ( !is_array( $remove ) ) {
		$remove = array( $remove );
	}
	return array_values( array_diff( $array, $remove ) );
}


/**
*	unpack_importbuddy()
*
*	Unpacks required files encoded in importbuddy.php into stand-alone files.
*
*	@return		null
*/
function unpack_importbuddy() {
	if ( !is_writable( ABSPATH . '/' ) ) {
		echo 'Error #224834. This directory is not write enabled. Please verify write permissions to continue.';
		die();
	} else {
		$unpack_file = '';
		
		$handle = @fopen( ABSPATH . '/importbuddy.php', 'r' );
		if ( $handle ) {
			while ( ( $buffer = fgets( $handle ) ) !== false ) {
				if ( substr( $buffer, 0, 11 ) == '###PACKDATA' ) {
					$packdata_commands = explode( ',', trim( $buffer ) );
					array_shift( $packdata_commands );
					
					if ( $packdata_commands[0] == 'BEGIN' ) {
						// Start packed data.
					} elseif ( $packdata_commands[0] == 'FILE_START' ) {
						$unpack_file = $packdata_commands[2];
					} elseif ( $packdata_commands[0] == 'FILE_END' ) {
						$unpack_file = '';
					} elseif ( $packdata_commands[0] == 'END' ) {
						return;
					}
				} else {
					if ( $unpack_file != '' ) {
						if ( !is_dir( dirname( ABSPATH . $unpack_file ) ) ) {
							mkdir_recursive( dirname( ABSPATH . $unpack_file ) );
						}
						file_put_contents( ABSPATH . $unpack_file, trim( base64_decode( $buffer ) ) );
					}
				}
			}
			if ( !feof( $handle ) ) {
				echo "Error: unexpected fgets() fail\n";
			}
			fclose( $handle );
		}
	}
}
die();
?>